@@ -3,13 +3,13 @@
 # Picard, the next-generation MusicBrainz tagger
 #
 # Copyright (C) 2006-2008, 2011 Lukáš Lalinský
-# Copyright (C) 2009-2011, 2015, 2018-2020 Philipp Wolfer
+# Copyright (C) 2009-2011, 2015, 2018-2021 Philipp Wolfer
 # Copyright (C) 2011 Johannes Weißl
 # Copyright (C) 2011-2014 Wieland Hoffmann
 # Copyright (C) 2012-2013 Michael Wiencek
 # Copyright (C) 2013 Calvin Walton
 # Copyright (C) 2013 Frederik “Freso” S. Olesen
-# Copyright (C) 2013-2014, 2018-2019 Laurent Monin
+# Copyright (C) 2013-2014, 2018-2021 Laurent Monin
 # Copyright (C) 2014-2015 Sophist-UK
 # Copyright (C) 2016-2018 Sambhav Kothari
 # Copyright (C) 2019 Reinaldo Antonio Camargo Rauch
@@ -36,10 +36,8 @@ from mutagen.mp4 import (
     MP4Cover,
 )
 
-from picard import (
-    config,
-    log,
-)
+from picard import log
+from picard.config import get_config
 from picard.coverart.image import (
     CoverArtImageError,
     TagCoverArtImage,
@@ -56,6 +54,7 @@ def _add_text_values_to_metadata(metadata, name, values):
 
 
 _VALID_KEY_CHARS = re.compile('^[\x00-\xff]+$')
+UNSUPPORTED_TAGS = {'r128_album_gain', 'r128_track_gain'}
 
 
 def _is_valid_key(key):
@@ -66,7 +65,7 @@ def _is_valid_key(key):
 
 
 class MP4File(File):
-    EXTENSIONS = [".m4a", ".m4b", ".m4p", ".m4v", ".mp4"]
+    EXTENSIONS = [".m4a", ".m4b", ".m4p", ".m4v", ".m4r", ".mp4"]
     NAME = "MPEG-4 Audio"
     _File = MP4
 
@@ -80,8 +79,9 @@ class MP4File(File):
         "\xa9day": "date",
         "\xa9gen": "genre",
         "\xa9lyr": "lyrics",
-        "\xa9cmt": "comment:",
+        "\xa9cmt": "comment",
         "\xa9too": "encodedby",
+        "\xa9dir": "director",
         "cprt": "copyright",
         "soal": "albumsort",
         "soaa": "albumartistsort",
@@ -94,14 +94,14 @@ class MP4File(File):
         "\xa9mvn": "movement",
         "\xa9wrk": "work",
     }
-    __r_text_tags = dict([(v, k) for k, v in __text_tags.items()])
+    __r_text_tags = {v: k for k, v in __text_tags.items()}
 
     __bool_tags = {
         "pcst": "podcast",
         "cpil": "compilation",
         "pgap": "gapless",
     }
-    __r_bool_tags = dict([(v, k) for k, v in __bool_tags.items()])
+    __r_bool_tags = {v: k for k, v in __bool_tags.items()}
 
     __int_tags = {
         "tmpo": "bpm",
@@ -109,7 +109,7 @@ class MP4File(File):
         "\xa9mvc": "movementtotal",
         "shwm": "showmovement",
     }
-    __r_int_tags = dict([(v, k) for k, v in __int_tags.items()])
+    __r_int_tags = {v: k for k, v in __int_tags.items()}
 
     __freeform_tags = {
         "----:com.apple.iTunes:MusicBrainz Track Id": "musicbrainz_recordingid",
@@ -152,7 +152,7 @@ class MP4File(File):
         "----:com.apple.iTunes:WORK": "work",
         "----:com.apple.iTunes:initialkey": "key",
     }
-    __r_freeform_tags = dict([(v, k) for k, v in __freeform_tags.items()])
+    __r_freeform_tags = {v: k for k, v in __freeform_tags.items()}
 
     # Tags to load case insensitive. Case is preserved, but the specified case
     # is written if it is unset.
@@ -165,7 +165,7 @@ class MP4File(File):
         "replaygain_track_range": "----:com.apple.iTunes:REPLAYGAIN_TRACK_RANGE",
         "replaygain_reference_loudness": "----:com.apple.iTunes:REPLAYGAIN_REFERENCE_LOUDNESS",
     }
-    __freeform_tags_ci = dict([(b.lower(), a) for a, b in __r_freeform_tags_ci.items()])
+    __freeform_tags_ci = {b.lower(): a for a, b in __r_freeform_tags_ci.items()}
 
     __other_supported_tags = ("discnumber", "tracknumber",
                               "totaldiscs", "totaltracks")
@@ -216,8 +216,7 @@ class MP4File(File):
                     log.debug('disk is invalid, ignoring')
             elif name == "covr":
                 for value in values:
-                    if value.imageformat not in (value.FORMAT_JPEG,
-                                                 value.FORMAT_PNG):
+                    if value.imageformat not in {value.FORMAT_JPEG, value.FORMAT_PNG}:
                         continue
                     try:
                         coverartimage = TagCoverArtImage(
@@ -247,17 +246,23 @@ class MP4File(File):
 
     def _save(self, filename, metadata):
         log.debug("Saving file %r", filename)
+        config = get_config()
         file = MP4(encode_filename(self.filename))
         if file.tags is None:
             file.add_tags()
         tags = file.tags
 
         if config.setting["clear_existing_tags"]:
+            cover = tags.get('covr') if config.setting['preserve_images'] else None
             tags.clear()
+            if cover:
+                tags['covr'] = cover
 
         for name, values in metadata.rawitems():
             if name.startswith('lyrics:'):
                 name = 'lyrics'
+            if name == 'comment:':
+                name = 'comment'
             if name in self.__r_text_tags:
                 tags[self.__r_text_tags[name]] = values
             elif name in self.__r_bool_tags:
@@ -331,19 +336,17 @@ class MP4File(File):
         for tag in metadata.deleted_tags:
             real_name = self._get_tag_name(tag)
             if real_name and real_name in tags:
-                if tag not in ("totaltracks", "totaldiscs"):
+                if tag not in {"totaltracks", "totaldiscs"}:
                     del tags[real_name]
 
     @classmethod
     def supports_tag(cls, name):
-        unsupported_tags = ['r128_album_gain', 'r128_track_gain']
-        return ((name
-                 and not name.startswith("~")
-                 and name not in unsupported_tags
-                 and not (name.startswith('comment:') and len(name) > 9)
-                 and not name.startswith('performer:')
-                 and _is_valid_key(name))
-                or name in ('~length'))
+        return (name
+                and not name.startswith("~")
+                and name not in UNSUPPORTED_TAGS
+                and not (name.startswith('comment:') and len(name) > 9)
+                and not name.startswith('performer:')
+                and _is_valid_key(name))
 
     def _get_tag_name(self, name):
         if name.startswith('lyrics:'):
@@ -360,9 +363,9 @@ class MP4File(File):
             return self.__r_freeform_tags_ci[name]
         elif name == "musicip_fingerprint":
             return "----:com.apple.iTunes:fingerprint"
-        elif name in ("tracknumber", "totaltracks"):
+        elif name in {"tracknumber", "totaltracks"}:
             return "trkn"
-        elif name in ("discnumber", "totaldiscs"):
+        elif name in {"discnumber", "totaldiscs"}:
             return "disk"
         elif self.supports_tag(name) and name not in self.__other_supported_tags:
             name = self.__casemap.get(name, name)
