@@ -3,9 +3,9 @@
 # Picard, the next-generation MusicBrainz tagger
 #
 # Copyright (C) 2012 Frederik “Freso” S. Olesen
-# Copyright (C) 2013-2014, 2018-2019 Laurent Monin
+# Copyright (C) 2013-2014, 2018-2021 Laurent Monin
 # Copyright (C) 2017 Sambhav Kothari
-# Copyright (C) 2017-2019 Philipp Wolfer
+# Copyright (C) 2017-2021 Philipp Wolfer
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -38,46 +38,60 @@ from picard.const.sys import (
 builtins.__dict__['N_'] = lambda a: a
 
 
+if IS_WIN:
+    from ctypes import windll
+
+    def _init_default_locale():
+        try:
+            current_locale = locale.windows_locale[windll.kernel32.GetUserDefaultUILanguage()]
+            return locale.setlocale(locale.LC_ALL, current_locale)
+        except KeyError:
+            return locale.setlocale(locale.LC_ALL, '')
+
+elif IS_MACOS:
+    import Foundation
+
+    def _init_default_locale():
+        defaults = Foundation.NSUserDefaults.standardUserDefaults()
+        current_locale = defaults.objectForKey_('AppleLanguages')[0]
+        current_locale = current_locale.replace('-', '_')
+        locale.setlocale(locale.LC_ALL, current_locale)
+        return current_locale
+
+else:
+    def _init_default_locale():
+        locale.setlocale(locale.LC_ALL, '')
+        return '.'.join(locale.getlocale(locale.LC_MESSAGES))
+
+
+def _try_set_locale(language):
+    # Try setting the locale with different or no encoding
+    for encoding in (locale.getpreferredencoding(), 'UTF-8', None):
+        if encoding:
+            current_locale = locale.normalize(language + '.' + encoding)
+        else:
+            current_locale = language
+        try:
+            locale.setlocale(locale.LC_ALL, current_locale)
+            return current_locale
+        except locale.Error:
+            continue
+    locale.setlocale(locale.LC_ALL, '')  # Ensure some locale settings are defined
+    return language  # Just return the language, so at least UI translation works
+
+
 def setup_gettext(localedir, ui_language=None, logger=None):
     """Setup locales, load translations, install gettext functions."""
     if not logger:
         logger = lambda *a, **b: None  # noqa: E731
     current_locale = ''
-    if ui_language:
-        try:
-            current_locale = locale.normalize(ui_language + '.' + locale.getpreferredencoding())
-            locale.setlocale(locale.LC_ALL, current_locale)
-        except Exception as e:
-            logger(e)
-    else:
-        if IS_WIN:
-            from ctypes import windll
-            try:
-                current_locale = locale.windows_locale[windll.kernel32.GetUserDefaultUILanguage()]
-                current_locale += '.' + locale.getpreferredencoding()
-                locale.setlocale(locale.LC_ALL, current_locale)
-            except KeyError:
-                try:
-                    current_locale = locale.setlocale(locale.LC_ALL, '')
-                except Exception as e:
-                    logger(e)
-            except Exception as e:
-                logger(e)
-        elif IS_MACOS:
-            try:
-                import Foundation
-                defaults = Foundation.NSUserDefaults.standardUserDefaults()
-                current_locale = defaults.objectForKey_('AppleLanguages')[0]
-                current_locale = current_locale.replace('-', '_')
-                locale.setlocale(locale.LC_ALL, current_locale)
-            except Exception as e:
-                logger(e)
+    try:
+        if ui_language:
+            current_locale = _try_set_locale(ui_language)
         else:
-            try:
-                locale.setlocale(locale.LC_ALL, '')
-                current_locale = '.'.join(locale.getlocale(locale.LC_MESSAGES))
-            except Exception as e:
-                logger(e)
+            current_locale = _init_default_locale()
+    except Exception as e:
+        logger(e)
     os.environ['LANGUAGE'] = os.environ['LANG'] = current_locale
     QLocale.setDefault(QLocale(current_locale))
     logger("Using locale %r", current_locale)
@@ -88,7 +102,7 @@ def setup_gettext(localedir, ui_language=None, logger=None):
         trans_countries = gettext.translation("picard-countries", localedir)
         logger("Loading gettext translation (picard-attributes), localedir=%r", localedir)
         trans_attributes = gettext.translation("picard-attributes", localedir)
-    except IOError as e:
+    except OSError as e:
         logger(e)
         trans = gettext.NullTranslations()
         trans_countries = gettext.NullTranslations()
