@@ -14,14 +14,17 @@
 # Copyright (C) 2011-2013 Michael Wiencek
 # Copyright (C) 2012 Your Name
 # Copyright (C) 2012-2013 Wieland Hoffmann
-# Copyright (C) 2013-2014, 2016, 2018-2019 Laurent Monin
-# Copyright (C) 2013-2014, 2017 Sophist-UK
+# Copyright (C) 2013-2014, 2016, 2018-2021 Laurent Monin
+# Copyright (C) 2013-2014, 2017, 2020 Sophist-UK
 # Copyright (C) 2016 Rahul Raturi
 # Copyright (C) 2016 Simon Legner
 # Copyright (C) 2016 Suhas
 # Copyright (C) 2016-2017 Sambhav Kothari
 # Copyright (C) 2018 Vishal Choudhary
-# Copyright (C) 2020 Gabriel Ferreira
+# Copyright (C) 2020-2021 Gabriel Ferreira
+# Copyright (C) 2021 Bob Swift
+# Copyright (C) 2021 Louis Sautier
+# Copyright (C) 2021 Petit Minion
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -44,6 +47,7 @@ from heapq import (
     heappop,
     heappush,
 )
+from locale import strxfrm
 
 from PyQt5 import (
     QtCore,
@@ -74,6 +78,7 @@ from picard.track import (
 )
 from picard.util import (
     icontheme,
+    iter_files_from_objects,
     natsort,
     normpath,
     restore_method,
@@ -139,16 +144,14 @@ def get_match_color(similarity, basecolor):
     c1 = (basecolor.red(), basecolor.green(), basecolor.blue())
     c2 = (223, 125, 125)
     return QtGui.QColor(
-        c2[0] + (c1[0] - c2[0]) * similarity,
-        c2[1] + (c1[1] - c2[1]) * similarity,
-        c2[2] + (c1[2] - c2[2]) * similarity)
+        int(c2[0] + (c1[0] - c2[0]) * similarity),
+        int(c2[1] + (c1[1] - c2[1]) * similarity),
+        int(c2[2] + (c1[2] - c2[2]) * similarity))
 
 
 class MainPanel(QtWidgets.QSplitter):
 
-    options = [
-        Option("persist", "splitter_state", QtCore.QByteArray()),
-    ]
+    options = []
 
     columns = [
         (N_('Title'), 'title'),
@@ -165,8 +168,10 @@ class MainPanel(QtWidgets.QSplitter):
         (N_('Media'), 'media'),
         (N_('Genre'), 'genre'),
         (N_('Fingerprint status'), '~fingerprint'),
+        (N_('AcousticBrainz status'), '~acousticbrainz'),
         (N_('Date'), 'date'),
         (N_('Original Release Date'), 'originaldate'),
+        (N_('Cover'), 'covercount'),
     ]
 
     _column_indexes = {column[1]: i for i, column in enumerate(columns)}
@@ -176,6 +181,7 @@ class MainPanel(QtWidgets.QSplitter):
     DISCNUMBER_COLUMN = _column_indexes['discnumber']
     LENGTH_COLUMN = _column_indexes['~length']
     FINGERPRINT_COLUMN = _column_indexes['~fingerprint']
+    ACOUSTICBRAINZ_COLUMN = _column_indexes['~acousticbrainz']
 
     NAT_SORT_COLUMNS = [
         _column_indexes['title'],
@@ -233,16 +239,9 @@ class MainPanel(QtWidgets.QSplitter):
         tab_order(prev, after)
 
     def save_state(self):
-        config = get_config()
-        config.persist["splitter_state"] = self.saveState()
         for view in self._views:
             view.save_state()
 
-    @restore_method
-    def restore_state(self):
-        config = get_config()
-        self.restoreState(config.persist["splitter_state"])
-
     def create_icons(self):
         if hasattr(QtWidgets.QStyle, 'SP_DirIcon'):
             ClusterItem.icon_dir = self.style().standardIcon(QtWidgets.QStyle.SP_DirIcon)
@@ -264,6 +263,7 @@ class MainPanel(QtWidgets.QSplitter):
         FileItem.icon_saved = QtGui.QIcon(":/images/track-saved.png")
         FileItem.icon_fingerprint = icontheme.lookup('fingerprint', icontheme.ICON_SIZE_MENU)
         FileItem.icon_fingerprint_gray = icontheme.lookup('fingerprint-gray', icontheme.ICON_SIZE_MENU)
+        FileItem.icon_acousticbrainz = icontheme.lookup('acousticbrainz', icontheme.ICON_SIZE_MENU)
         FileItem.match_icons = [
             QtGui.QIcon(":/images/match-50.png"),
             QtGui.QIcon(":/images/match-60.png"),
@@ -318,12 +318,6 @@ class MainPanel(QtWidgets.QSplitter):
         for view in self._views:
             view.setSortingEnabled(sort)
 
-    def collapse_clusters(self, collapse=True):
-        if collapse:
-            self._views[0].collapseAll()
-        else:
-            self._views[0].expandAll()
-
     def select_object(self, obj):
         item = obj.item
         for view in self._views:
@@ -333,12 +327,12 @@ class MainPanel(QtWidgets.QSplitter):
                 break
 
 
-def paint_fingerprint_icon(painter, rect, icon):
+def paint_column_icon(painter, rect, icon):
     if not icon:
         return
     size = COLUMN_ICON_SIZE
     padding_h = COLUMN_ICON_BORDER
-    padding_v = (rect.height() - size) / 2
+    padding_v = (rect.height() - size) // 2
     target_rect = QtCore.QRect(rect.x() + padding_h, rect.y() + padding_v, size, size)
     painter.drawPixmap(target_rect, icon.pixmap(size, size))
 
@@ -371,7 +365,7 @@ class ConfigurableColumnsHeader(TristateSortHeaderView):
             if self.sectionSize(column) == 0:
                 self.resizeSection(column, self.defaultSectionSize())
             self._visible_columns.add(column)
-            if column == MainPanel.FINGERPRINT_COLUMN:
+            if column in {MainPanel.FINGERPRINT_COLUMN, MainPanel.ACOUSTICBRAINZ_COLUMN}:
                 self.setSectionResizeMode(column, QtWidgets.QHeaderView.Fixed)
                 self.resizeSection(column, COLUMN_ICON_SIZE)
             else:
@@ -383,10 +377,6 @@ class ConfigurableColumnsHeader(TristateSortHeaderView):
         for i, column in enumerate(MainPanel.columns):
             self.show_column(i, i in columns)
 
-    @property
-    def visible_columns(self):
-        return self._visible_columns
-
     def contextMenuEvent(self, event):
         menu = QtWidgets.QMenu(self)
         parent = self.parent()
@@ -424,19 +414,24 @@ class ConfigurableColumnsHeader(TristateSortHeaderView):
             painter.save()
             super().paintSection(painter, rect, index)
             painter.restore()
-            paint_fingerprint_icon(painter, rect, FileItem.icon_fingerprint_gray)
+            paint_column_icon(painter, rect, FileItem.icon_fingerprint_gray)
+        elif index == MainPanel.ACOUSTICBRAINZ_COLUMN:
+            painter.save()
+            super().paintSection(painter, rect, index)
+            painter.restore()
+            paint_column_icon(painter, rect, FileItem.icon_acousticbrainz)
         else:
             super().paintSection(painter, rect, index)
 
     def on_sort_indicator_changed(self, index, order):
-        if index == MainPanel.FINGERPRINT_COLUMN:
+        if index in {MainPanel.FINGERPRINT_COLUMN, MainPanel.ACOUSTICBRAINZ_COLUMN}:
             self.setSortIndicator(-1, QtCore.Qt.AscendingOrder)
 
     def lock(self, is_locked):
         super().lock(is_locked)
-        fingerprint_column_index = MainPanel.FINGERPRINT_COLUMN
-        if not self.is_locked and self.count() > fingerprint_column_index:
-            self.setSectionResizeMode(fingerprint_column_index, QtWidgets.QHeaderView.Fixed)
+        for column_index in {MainPanel.FINGERPRINT_COLUMN, MainPanel.ACOUSTICBRAINZ_COLUMN}:
+            if not self.is_locked and self.count() > column_index:
+                self.setSectionResizeMode(column_index, QtWidgets.QHeaderView.Fixed)
 
 
 class BaseTreeView(QtWidgets.QTreeWidget):
@@ -448,7 +443,7 @@ class BaseTreeView(QtWidgets.QTreeWidget):
         self.panel = parent
         # Should multiple files dropped be assigned to tracks sequentially?
         self._move_to_multi_tracks = True
-        self.setHeaderLabels([_(h) if n != '~fingerprint' else ''
+        self.setHeaderLabels([_(h) if n not in {'~fingerprint', '~acousticbrainz'} else ''
                               for h, n in MainPanel.columns])
         self.restore_state()
 
@@ -498,6 +493,8 @@ class BaseTreeView(QtWidgets.QTreeWidget):
             if can_view_info:
                 menu.addAction(self.window.view_info_action)
             menu.addAction(self.window.browser_lookup_action)
+            if self.window.submit_cluster_action:
+                menu.addAction(self.window.submit_cluster_action)
             menu.addSeparator()
             menu.addAction(self.window.autotag_action)
             menu.addAction(self.window.analyze_action)
@@ -518,6 +515,10 @@ class BaseTreeView(QtWidgets.QTreeWidget):
             menu.addAction(self.window.play_file_action)
             menu.addAction(self.window.open_folder_action)
             menu.addAction(self.window.browser_lookup_action)
+            if self.window.submit_file_as_recording_action:
+                menu.addAction(self.window.submit_file_as_recording_action)
+            if self.window.submit_file_as_release_action:
+                menu.addAction(self.window.submit_file_as_release_action)
             menu.addSeparator()
             menu.addAction(self.window.autotag_action)
             menu.addAction(self.window.analyze_action)
@@ -748,7 +749,7 @@ class BaseTreeView(QtWidgets.QTreeWidget):
                     files.append(file)
                 else:
                     new_paths.append(filename)
-            elif url.scheme() in ("http", "https"):
+            elif url.scheme() in {"http", "https"}:
                 file_lookup = tagger.get_file_lookup()
                 file_lookup.mbid_lookup(url.path(), browser_fallback=False)
         if files:
@@ -785,10 +786,12 @@ class BaseTreeView(QtWidgets.QTreeWidget):
         # application/picard.album-list
         albums = data.data("application/picard.album-list")
         if albums:
-            albums = [self.tagger.load_album(id) for id in bytes(albums).decode().split("\n")]
+            album_ids = bytes(albums).decode().split("\n")
+            log.debug("Dropped albums = %r", album_ids)
+            files = iter_files_from_objects(self.tagger.load_album(id) for id in album_ids)
             # Use QTimer.singleShot to run expensive processing outside of the drop handler.
-            QtCore.QTimer.singleShot(0, partial(self.tagger.move_files,
-                self.tagger.get_files_from_objects(albums), target))
+            move_files = partial(self.tagger.move_files, files, target)
+            QtCore.QTimer.singleShot(0, move_files)
             handled = True
         self._move_to_multi_tracks = True  # Reset for next drop
         return handled
@@ -811,7 +814,7 @@ class BaseTreeView(QtWidgets.QTreeWidget):
             cluster_item.add_files(cluster.files)
 
     def moveCursor(self, action, modifiers):
-        if action in (QtWidgets.QAbstractItemView.MoveUp, QtWidgets.QAbstractItemView.MoveDown):
+        if action in {QtWidgets.QAbstractItemView.MoveUp, QtWidgets.QAbstractItemView.MoveDown}:
             item = self.currentItem()
             if item and not item.isSelected():
                 self.setCurrentItem(item)
@@ -896,15 +899,17 @@ class TreeItem(QtWidgets.QTreeWidgetItem):
         ]:
             self.setTextAlignment(column, QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
         self.setSizeHint(MainPanel.FINGERPRINT_COLUMN, ICON_SIZE)
+        self.setSizeHint(MainPanel.ACOUSTICBRAINZ_COLUMN, ICON_SIZE)
 
     def setText(self, column, text):
         self._sortkeys[column] = None
         return super().setText(column, text)
 
     def __lt__(self, other):
-        if not self.sortable:
+        tree_widget = self.treeWidget()
+        if not self.sortable or not tree_widget:
             return False
-        column = self.treeWidget().sortColumn()
+        column = tree_widget.sortColumn()
         return self.sortkey(column) < other.sortkey(column)
 
     def sortkey(self, column):
@@ -915,9 +920,9 @@ class TreeItem(QtWidgets.QTreeWidgetItem):
         if column == MainPanel.LENGTH_COLUMN:
             sortkey = self.obj.metadata.length or 0
         elif column in MainPanel.NAT_SORT_COLUMNS:
-            sortkey = natsort.natkey(self.text(column).lower())
+            sortkey = natsort.natkey(self.text(column))
         else:
-            sortkey = self.text(column).lower()
+            sortkey = strxfrm(self.text(column))
         self._sortkeys[column] = sortkey
         return sortkey
 
@@ -991,11 +996,13 @@ class AlbumItem(TreeItem):
                 # insertChildren behaves differently if sorting is disabled / enabled, which results
                 # in different sort order of tracks in unsorted state. As we sort the tracks later
                 # anyway make sure sorting is disabled here.
-                tree_view = self.treeWidget()
-                sorting_enabled = tree_view.isSortingEnabled()
-                tree_view.setSortingEnabled(False)
+                tree_widget = self.treeWidget()
+                if tree_widget:
+                    sorting_enabled = tree_widget.isSortingEnabled()
+                    tree_widget.setSortingEnabled(False)
                 self.insertChildren(oldnum, items)
-                tree_view.setSortingEnabled(sorting_enabled)
+                if tree_widget:
+                    tree_widget.setSortingEnabled(sorting_enabled)
                 for item in items:  # Update after insertChildren so that setExpanded works
                     item.update(update_album=False)
         if album.errors:
@@ -1033,7 +1040,10 @@ class AlbumItem(TreeItem):
 class NatAlbumItem(AlbumItem):
     def __lt__(self, other):
         # Always show NAT entry on top
-        order = self.treeWidget().header().sortIndicatorOrder()
+        tree_widget = self.treeWidget()
+        if not tree_widget:
+            return True
+        order = tree_widget.header().sortIndicatorOrder()
         return order == QtCore.Qt.AscendingOrder
 
 
@@ -1053,10 +1063,15 @@ class TrackItem(TreeItem):
             fingerprint_icon, fingerprint_tooltip = FileItem.decide_fingerprint_icon_info(file)
             self.setToolTip(MainPanel.FINGERPRINT_COLUMN, fingerprint_tooltip)
             self.setIcon(MainPanel.FINGERPRINT_COLUMN, fingerprint_icon)
+            ab_icon, ab_tooltip = FileItem.decide_ab_icon_info(file)
+            self.setToolTip(MainPanel.ACOUSTICBRAINZ_COLUMN, ab_tooltip)
+            self.setIcon(MainPanel.ACOUSTICBRAINZ_COLUMN, ab_icon)
         else:
             self.setToolTip(MainPanel.TITLE_COLUMN, "")
             self.setToolTip(MainPanel.FINGERPRINT_COLUMN, "")
             self.setIcon(MainPanel.FINGERPRINT_COLUMN, QtGui.QIcon())
+            self.setToolTip(MainPanel.ACOUSTICBRAINZ_COLUMN, "")
+            self.setIcon(MainPanel.ACOUSTICBRAINZ_COLUMN, QtGui.QIcon())
             if track.ignored_for_completeness():
                 color = TreeItem.text_color_secondary
             else:
@@ -1112,6 +1127,9 @@ class FileItem(TreeItem):
         fingerprint_icon, fingerprint_tooltip = FileItem.decide_fingerprint_icon_info(file)
         self.setToolTip(MainPanel.FINGERPRINT_COLUMN, fingerprint_tooltip)
         self.setIcon(MainPanel.FINGERPRINT_COLUMN, fingerprint_icon)
+        ab_icon, ab_tooltip = FileItem.decide_ab_icon_info(file)
+        self.setToolTip(MainPanel.ACOUSTICBRAINZ_COLUMN, ab_tooltip)
+        self.setIcon(MainPanel.ACOUSTICBRAINZ_COLUMN, ab_icon)
         color = FileItem.file_colors[file.state]
         bgcolor = get_match_color(file.similarity, TreeItem.base_color)
         for i, column in enumerate(MainPanel.columns):
@@ -1168,3 +1186,22 @@ class FileItem(TreeItem):
             icon = QtGui.QIcon()
             tooltip = _('No fingerprint was calculated for this file, use "Scan" or "Generate AcoustID Fingerprints" to calculate the fingerprint.')
         return (icon, tooltip)
+
+    @staticmethod
+    def decide_ab_icon_info(file):
+        if file.acousticbrainz_error:
+            icon = TrackItem.icon_error
+            tooltip = _('Extraction or submission of Acoustic features failed')
+        elif file.acousticbrainz_is_duplicate:
+            icon = FileItem.icon_saved
+            if file.acousticbrainz_features_file is None:
+                tooltip = _('The server already has the features of this file')
+            else:
+                tooltip = _('Features have already been submitted')
+        else:
+            icon = FileItem.icon_file_pending
+            if file.acousticbrainz_features_file is None:
+                tooltip = _('No acoustic features were extracted from this file. Use "Submit Acoustic features" to extract and submit them')
+            else:
+                tooltip = _('Unsubmitted features')
+        return icon, tooltip
