@@ -3,12 +3,13 @@
 # Picard, the next-generation MusicBrainz tagger
 #
 # Copyright (C) 2007 Lukáš Lalinský
-# Copyright (C) 2008-2009, 2019-2020 Philipp Wolfer
+# Copyright (C) 2008-2009, 2019-2021 Philipp Wolfer
 # Copyright (C) 2012-2013 Michael Wiencek
-# Copyright (C) 2013-2014, 2018-2019 Laurent Monin
+# Copyright (C) 2013-2014, 2018-2021 Laurent Monin
 # Copyright (C) 2014 Sophist-UK
 # Copyright (C) 2016, 2018 Sambhav Kothari
 # Copyright (C) 2018 Wieland Hoffmann
+# Copyright (C) 2021 Gabriel Ferreira
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -26,7 +27,9 @@
 
 
 from functools import partial
+import logging
 import os
+import re
 
 from PyQt5 import (
     QtCore,
@@ -39,7 +42,10 @@ from picard.config import (
     IntOption,
     get_config,
 )
-from picard.util import reconnect
+from picard.util import (
+    reconnect,
+    wildcards_to_regex_pattern,
+)
 
 from picard.ui import (
     FONT_FAMILY_MONOSPACE,
@@ -78,6 +84,7 @@ class LogViewCommon(LogViewDialog):
         self.textCursor.movePosition(QtGui.QTextCursor.Start)
 
     def closeEvent(self, event):
+        self.save_geometry()
         event.ignore()
         self.hide()
 
@@ -118,19 +125,13 @@ class Highlighter(QtGui.QSyntaxHighlighter):
 
         self.fmt = QtGui.QTextCharFormat()
         self.fmt.setBackground(QtCore.Qt.lightGray)
-
-        self.reg = QtCore.QRegExp()
-        self.reg.setPatternSyntax(QtCore.QRegExp.Wildcard)
-        self.reg.setCaseSensitivity(QtCore.Qt.CaseInsensitive)
-        self.reg.setPattern(string)
+        self.reg = re.compile(wildcards_to_regex_pattern(string), re.IGNORECASE)
 
     def highlightBlock(self, text):
-        expression = self.reg
-        index = expression.indexIn(text)
-        while index >= 0:
-            length = expression.matchedLength()
+        for match in self.reg.finditer(text):
+            index = match.start()
+            length = match.end() - match.start()
             self.setFormat(index, length, self.fmt)
-            index = expression.indexIn(text, index + length)
 
 
 class VerbosityMenu(QtWidgets.QMenu):
@@ -156,7 +157,7 @@ class VerbosityMenu(QtWidgets.QMenu):
 class LogView(LogViewCommon):
 
     options = [
-        IntOption("setting", "log_verbosity", log.VERBOSITY_DEFAULT),
+        IntOption("setting", "log_verbosity", logging.WARNING),
     ]
 
     def __init__(self, parent=None):
@@ -209,6 +210,8 @@ class LogView(LogViewCommon):
         self.hbox.addWidget(self.save_log_as_button)
         self.save_log_as_button.clicked.connect(self._save_log_as_do)
 
+        self._prev_logitem_level = logging.NOTSET
+
     def _clear_highlight_do(self):
         self.highlight_text.setText('')
         self.highlight_button.setEnabled(False)
@@ -276,6 +279,11 @@ class LogView(LogViewCommon):
         self.highlight_text.setFocus(QtCore.Qt.OtherFocusReason)
         super().show()
 
+    def display(self, clear=False):
+        if clear:
+            self._prev_logitem_level = logging.NOTSET
+        super().display(clear=clear)
+
     def _clear_log_do(self):
         reply = QtWidgets.QMessageBox.question(
             self,
@@ -294,10 +302,10 @@ class LogView(LogViewCommon):
     def _add_entry(self, logitem):
         if not self.is_shown(logitem):
             return
-        fmt = self.textCursor.blockCharFormat()
-        self.textCursor.setBlockCharFormat(self._format(logitem.level))
+        if self._prev_logitem_level != logitem.level:
+            self.textCursor.setBlockCharFormat(self._format(logitem.level))
+            self._prev_logitem_level = logitem.level
         super()._add_entry(logitem)
-        self.textCursor.setBlockCharFormat(fmt)
 
     def _set_verbosity(self, level):
         self.verbosity = level
