@@ -6,14 +6,15 @@
 # Copyright (C) 2006-2007, 2012 Lukáš Lalinský
 # Copyright (C) 2011-2014 Michael Wiencek
 # Copyright (C) 2012 Nikolai Prokoschenko
-# Copyright (C) 2013-2014 Sophist-UK
-# Copyright (C) 2013-2014, 2017-2019 Laurent Monin
+# Copyright (C) 2013-2014, 2017-2021 Laurent Monin
+# Copyright (C) 2013-2014, 2021 Sophist-UK
 # Copyright (C) 2015 Ohm Patel
 # Copyright (C) 2015 Wieland Hoffmann
 # Copyright (C) 2015, 2018-2021 Philipp Wolfer
 # Copyright (C) 2016-2018 Sambhav Kothari
 # Copyright (C) 2018 Vishal Choudhary
-# Copyright (C) 2020 Gabriel Ferreira
+# Copyright (C) 2020 Felix Schwarz
+# Copyright (C) 2020-2021 Gabriel Ferreira
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -30,7 +31,10 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 
-from collections import defaultdict
+from collections import (
+    Counter,
+    defaultdict,
+)
 from functools import partial
 
 from PyQt5 import (
@@ -68,7 +72,7 @@ from picard.ui.edittagdialog import (
 
 
 class TagStatus:
-
+    NONE = 0
     NOCHANGE = 1
     ADDED = 2
     REMOVED = 4
@@ -84,7 +88,7 @@ class TagCounter(dict):
 
     def __init__(self, parent):
         self.parent = parent
-        self.counts = defaultdict(lambda: 0)
+        self.counts = Counter()
         self.different = set()
 
     def __getitem__(self, tag):
@@ -125,7 +129,7 @@ class TagDiff(object):
         self.tag_names = []
         self.new = TagCounter(self)
         self.orig = TagCounter(self)
-        self.status = defaultdict(lambda: 0)
+        self.status = defaultdict(lambda: TagStatus.NONE)
         self.objects = 0
         self.max_length_delta_ms = max_length_diff * 1000
 
@@ -284,9 +288,9 @@ class MetadataBox(QtWidgets.QTableWidget):
             return False
         item = self.itemFromIndex(index)
         if item.flags() & QtCore.Qt.ItemIsEditable and \
-           trigger in (QtWidgets.QAbstractItemView.DoubleClicked,
+           trigger in {QtWidgets.QAbstractItemView.DoubleClicked,
                        QtWidgets.QAbstractItemView.EditKeyPressed,
-                       QtWidgets.QAbstractItemView.AnyKeyPressed):
+                       QtWidgets.QAbstractItemView.AnyKeyPressed}:
             tag = self.tag_diff.tag_names[item.row()]
             values = self.tag_diff.new[tag]
             if len(values) > 1:
@@ -551,8 +555,8 @@ class MetadataBox(QtWidgets.QTableWidget):
             return None
 
         if new_selection or drop_album_caches:
-            self._single_file_album = len(set([file.metadata["album"] for file in files])) == 1
-            self._single_track_album = len(set([track.metadata["album"] for track in tracks])) == 1
+            self._single_file_album = len({file.metadata["album"] for file in files}) == 1
+            self._single_track_album = len({track.metadata["album"] for track in tracks}) == 1
 
         while not new_selection:  # Just an if with multiple exit points
             # If we are dealing with the same selection
@@ -585,14 +589,16 @@ class MetadataBox(QtWidgets.QTableWidget):
         top_tags = config.setting['metadatabox_top_tags']
         top_tags_set = set(top_tags)
 
+        settings = config.setting.as_dict()
+
         for file in files:
-            new_metadata = file.new_metadata
+            new_metadata = file.metadata
             orig_metadata = file.orig_metadata
-            tags = set(list(new_metadata.keys()) + list(orig_metadata.keys()))
+            tags = set(new_metadata) | set(orig_metadata)
 
             for name in filter(lambda x: not x.startswith("~") and file.supports_tag(x), tags):
-                new_values = new_metadata.getall(name)
-                orig_values = orig_metadata.getall(name)
+                new_values = file.format_specific_metadata(new_metadata, name, settings)
+                orig_values = file.format_specific_metadata(orig_metadata, name, settings)
 
                 if not clear_existing_tags and not new_values:
                     new_values = list(orig_values or [""])
